# cargo2nix

Build Rust packages inside Nix builds using Cargo directly.

This is the smallest possible "*2nix" tool for Rust and Cargo packaging.

## Usage

Install with [niv][niv]:

```shell
niv init
niv add cargo2nix -t https://euandre.org/static/attachments/cargo2nix.tar.gz
```

[niv]: https://github.com/nmattia/niv

Create a `default.nix` with:

```nix
let
  niv-sources = import ./nix/sources.nix;
  mozilla-overlay = import niv-sources.nixpkgs-mozilla;
  pkgs = import niv-sources.nixpkgs { overlays = [ mozilla-overlay ]; };
  src = pkgs.nix-gitignore.gitignoreSource [ ] ./.;
  cargo2nix = pkgs.callPackage niv-sources.cargo2nix {
    lockfile = ./Cargo.lock;
  };
in pkgs.stdenv.mkDerivation {
  inherit src;
  name = "cargo-test";
  buildInputs = [ pkgs.latest.rustChannels.nightly.rust ];
  phases = [ "unpackPhase" "buildPhase" ];
  buildPhase = ''
    # Setup dependencies path to satisfy Cargo
    mkdir .cargo/
    ln -s ${cargo2nix.env.cargo-config} .cargo/config
    ln -s ${cargo2nix.env.vendor} vendor

    # Run the tests
    cargo test
    touch $out
  '';
}
```

And than run `nix-build`:

```shell
nix-build
```

## Demo

You may also try the [demo repository]:

```shell
pushd "$(mktemp -d)"
wget -O- https://euandre.org/static/attachments/cargo2nix-demo.tar.gz |
	tar -xv
cd cargo2nix-demo/
nix-build
```

[demo repository]: https://euandre.org/static/attachments/cargo2nix-demo.tar.gz

## Limitations

The `cargo2nix` function (the contents of `default.nix`) only tries to
replicate the minimum environment and make it available so you can add it to a
build.

There is no support for 3rd party registries or Git repositories, yet. Patches
welcome.
