{ pkgs, lockfile }:
let
  resolved-deps =
    (builtins.fromTOML (builtins.readFile lockfile)).package;
in {
  inherit resolved-deps;
  env = {
    vendor = let
      make-checkout = { name, version, checksum ? null, ... }:
        if checksum == null then
          null
        else
          let
            crate = builtins.fetchurl {
              url = "https://crates.io/api/v1/crates/${name}/${version}/download";
              name = "download-${name}-${version}";
              sha256 = checksum;
            };
          in
            pkgs.runCommand "download-${name}" { } ''
              mkdir $out
              tar -xzf ${crate} -C $out
              echo '{"package":"${checksum}","files":{}}' > $out/${name}-${version}/.cargo-checksum.json
            '';
    in pkgs.symlinkJoin {
      name = "cargo2nix-vendored-dependencies";
      paths = map make-checkout resolved-deps;
    };
    cargo-config = pkgs.writeText "cargo-config" ''
      [source.crates-io]
      replace-with = "vendored-sources"

      [source.vendored-sources]
      directory = "vendor"
    '';
  };
}
